Document: bash
Title: Bash Manual Page
Author: Chet Ramey
Abstract:  Bash is an sh-compatible command language interpreter that executes
 commands read from the standard input or from a file.  Bash also
 incorporates useful features from the Korn and C shells (ksh and csh).
Section: Shells

Format: html
Index: /usr/share/doc/bash/bash.html
Files: /usr/share/doc/bash/bash.html

Format: pdf
Files: /usr/share/doc/bash/bash.pdf
